/*
 * Copyright 2021 styledart.dev - Mehmet Yaz
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

///
const simpleServer = TemplateGenerator([
  "bin",
  "data",
], {
  "bin/__projectName__.dart": """
import 'package:style_dart/style_dart.dart';

void main(List<String> arguments) {
  runService(MyServer());
}

class MyServer extends StatelessComponent {
  const MyServer({Key? key}) : super(key: key);

  @override
  Component build(BuildContext context) {
    return Server(
        rootName: "__projectName__",
        /// call http://localhost
        rootEndpoint: SimpleEndpoint.static("Hello World!"),
        dataAccess: DataAccess(SimpleDataAccess("__dataDir__")),
        children: [
          /// Call http://localhost/your_name eg. http://localhost/jack
          Route("{name}", root: SimpleEndpoint((r) {
            DataAccess.of(context).create(Create(
                request: r,
                collection: "greeters",
                data: {"id": r.arguments["name"]}));
            return r.response("Greet from \${r.arguments["name"]}");
          })),

          /// Call http://localhost/greeters
          Route("greeters", root: SimpleEndpoint((req) async {
            return req.response((await DataAccess.of(context).readList(
                    ReadMultiple(request: req, collection: "greeters")))
                .data);
          }))
        ]);
  }
}
""",
  "pubspec.yaml": """
name: __projectName__
description: A simple style data access server.
version: 1.0.0
publish_to: none

environment:
  sdk: '>=2.14.0 <3.0.0'

dependencies:
  style_dart:
    path: ../../../style

dev_dependencies:
  effective_dart: ^1.3.2
""",
  "analysis_options.yaml":
      "include: package:effective_dart/analysis_options.yaml"
});

///
class TemplateGenerator {
  ///
  const TemplateGenerator(this.directories, this.documents);

  ///
  final List<String> directories;

  ///
  final Map<String, String> documents;
}
